xquery version "1.0";

(: diese Option braucht man nur, wenn die Datei von exist aus aufgerufen wird 
   (damit das Ergebnis als HTML angezeigt wird) :)
(: declare option exist:serialize "method=xhtml media-type=text/html"; :)


(: ein HTML-div-Element soll alle Ergebnisse enthalten :)
<div>


    <!-- ÜBUNG 1: Titelliste aus einem Text generieren
    - Geben Sie mit XPath alle <TITLE>-Elemente aus (-> Achtung: im Browser sieht man die Ausgabe nur im Quelltext!)
    - Konstruieren Sie ein <ul>-Element und geben Sie darin die <TITLE>-Elemente aus (-> Konstruktor!)
    - Geben Sie für jedes-<TITLE>-Element ein <li>-Element mit dem Inhalt des <TITLE>-Elements zurück (-> For-Schleife! XPath-Funktion data() zur Ausgabe des Elementinhalts) -->
    <div>
        <h2>Titelliste</h2>
        <!-- ul: eine unsortiert HTML-Liste -->
        <ul>
           <!-- darin ein XQuery-Ausdruck (geschweifte Klammern!) 
            - For-Schleife: für jedes TITLE-Element wird ein Listeneintrag (li) mit dem Titel ausgegeben
            - die data()-Funktion brauchen wir, damit nur der WERT des Titelelements ausgegeben wird, nicht das ganze Element
            -->
            {for $title in (//TITLE)
            return <li>{data($title)}</li>}
        </ul>
    </div>


    <!-- ÜBUNG 2: Wie oft sprechen die Figuren?
    - Geben Sie eine Liste der verschiedenen Sprecher (<SPEAKER>) aus (-> For-Schleife!, XPath-Funktion distinct-values())
    - Sortieren Sie die Liste alphabetisch aufsteigend (-> Order by)
    - Geben Sie hinter jedem Sprecher die Anzahl seiner Reden (<SPEECH>) aus (-> XPath-Funktion count(), XPath-Bedingung)
    - Sortieren Sie die Liste nach Häufigkeit der Rede um -->
    <div>
        <ul>
            <!-- für jeden aller unterschiedlichen, im Text vorkommenden Sprecher... -->
            {for $speaker in distinct-values(//SPEAKER)
            (: wird die Anzahl seiner Redebeiträge gezählt und in der Variablen $anzahlSpeech gespeichert :)
            let $anzahlSpeech := for $speech in //SPEECH
                                 (: indem für alle SPEECH-Elemente geprüft wird... :)
                                where $speech/SPEAKER[. = $speaker]
                                (: ob sie vom aktuellen Sprecher (-> $speaker-Variable) stammen, falls ja, wird der Redebeitrag zurückgegeben :)
                                return $speech
            (: Sortierung der Liste nach Häufigkeit der Liste, absteigend :)
            order by count($anzahlSpeech) descending
            (: Rück-/Ausgabe eines Listenelements für jeden Sprecher mit Angabe der Redehäufigkeit :)
            return <li>{data($speaker)} ({count($anzahlSpeech)})</li>}
        </ul>
    </div>


    <!-- ÜBUNG 3: Suchen
    - Geben Sie alle Bühnenanweisungen (<STAGEDIR>) zurück, in denen Ophelia vorkommt (-> FLWOR; XPath-Funktion contains())
    - Geben Sie alle Zeilen (<LINE>) zurück, in denen „love“  (groß oder klein geschrieben) vorkommt (-> FLWOR; XPath-Funktion contains())
    - Geben Sie an, wie viele Treffer es sind (-> XPath-Funktion count())
    - Geben Sie an, aus welchem Akt (ACT) und welcher Szene (SCENE) die Treffer stammen (-> XPath-Achsen, XPath-Funktion count()) -->
    
    <div>
        <ul>
            {for $stagedir in //STAGEDIR
            where contains($stagedir, "OPHELIA")
            return <li>{data($stagedir)}</li>}
            <!-- Anmerkung:
                anstatt die Bedingung mit einem "Where"-Ausdruck zu formulieren, könnte man sie schon im "For"-Ausdruck direkt als XPath-Bedingung angeben:
                for $stagedir in //STAGEDIR[contains(., "OPHELIA")]
                return...
            -->
        </ul>
    </div>


    {(: geschweifte Klammern leiten hier den XQuery-Ausdruck ein, da wir uns innerhalb des äußeren, direkt konstruierten <div>s (siehe ganz oben) befinden! 
    - wir ermitteln zunächst die Zeilen, in denen "love" vorkommt und speichern das Ergebnis in einer Variablen, damit wir es später durch Aufruf der Variablen 
    gleich an mehreren Stellen ausgeben können :)
    let $lines := for $line in //LINE
                  (: Bedingung:  die in Kleinbuchstaben konvertierten Zeilen müssen "love" enthalten -> so ist Klein-/Großschreibung unerheblich :)
                  where contains(lower-case($line), "love")
                  return $line
    let $anzahlLines := count($lines)
    (: nun die Rückgabe :)
    return 
    <div>
        <h2>In welchen Zeilen kommt "love" vor?</h2>
        <p>{$anzahlLines} Treffer</p>
        <ol> <!-- eine nummerierte Liste (HTML) -->
            {for $line in $lines
            (: gib für jede Trefferzeile ein Listenelement aus :)
             return <li> { (: aus welchem Akt und welcher Szene stammt dieser Treffer? 
                         - Zähle die vorangegangenen Akte und 1 hinzu, da der Akt selbst, in dem sich die Zeile befindet, nicht mitgezählt wird :)
                         let $aktNr := count($line/preceding::ACT) + 1 
                         (: entsprechend für die Szene, hier mit der zusätzlichen Bedingung, dass die Nr. des Aktes mit der eben ermittelten übereinstimmen muss :)
                         let $szenenNr := count($line/preceding::SCENE[count(preceding::ACT) + 1 = $aktNr]) + 1
                         return
                            ("Akt ", $aktNr, "Szene ", $szenenNr, ": ")}
                       {data($line)}</li>}
        </ol>
    </div>}

    <!-- ÜBUNG 4: Fragen
    - Kommt Hamlet in allen Szenen vor? (-> every… in … satisfies)
    - Kommt jemand nur in einer Szene vor? (-> some… in… satisfies) -->
    <div>
        <h2>Kommt Hamlet in allen Szenen vor?</h2>
        {let $antwort := (: gilt für jedes Szeneelement... :)
                         every $szene in //SCENE
                         (: dass es ein SPEAKER-Element enthält, dessen Wert "HAMLET" ist? :)
                         satisfies ($szene//SPEAKER[. = "HAMLET"])
                         (: wenn ja, wird der Wahrheitswert "true" zurückgegeben, sonst "false" :)
         return (: If-Anweisung: wenn die die Antwort "true" ist, soll "Ja" ausgegeben werden, sonst "Nein" :)
                if ($antwort eq true())
                then "Ja"
                else "Nein"}
        <h2>Kommt jemand nur in einer Szene vor?</h2>
        {let $antwort := (: gilt für irgendeinen (mindestens einen) (-> some) der verschiedenen (-> distinct-values) vorkommenden Sprecher... :)
                         some $speaker in distinct-values(//SPEAKER)
                         (: dass die Anzahl der Szenen, in denen er vorkommt, 1 ist? .// im XPath-Ausdruck bedeutet hier, dass wir vom aktuellen Kontext aus (.) beliebig tief im Baum 
                         suchen, // allein würde von der Wurzel des Dokuments, also von ganz oben aus beliebig tief suchen! :)
                         satisfies count(//SCENE[.//SPEAKER]) = 1
         return if ($antwort eq true())
                    then "Ja"
                    else "Nein"}
    </div>
    
</div>

    



